using MonoTouch.UIKit;
using System;
using System.Drawing;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.iOSEdition;

namespace ZoomToCurrentCoordinatesUsingBing
{
    public partial class ZoomToCurrentCoordinatesUsingBingViewController : UIViewController
    {
        private MapView iOSMap;

        static ZoomToCurrentCoordinatesUsingBingViewController()
        {
            // This is the most important part for using latest iOS version.
            // I found the latest attached code doesn't include this one, that means you are still using the older version.
            // Please use the dlls in the dependencies folder in this sample package.
            PclShareEnvironment.Current = new iOSEditionPclShareEnviroment();
        }

        public override void ViewDidLoad()
        {
            base.ViewDidLoad();

            View.Frame = UIScreen.MainScreen.Bounds;
            View.BackgroundColor = UIColor.White;
            View.AutoresizingMask = UIViewAutoresizing.FlexibleWidth | UIViewAutoresizing.FlexibleHeight;

            iOSMap = new MapView(View.Frame);

            // We found you are using BingMaps as base map
            // So the map unit should follow bing's geography unit "meter".
            iOSMap.MapUnit = GeographyUnit.Meter;
            iOSMap.BackgroundColor = new UIColor(233, 229, 220, 200);
            iOSMap.ZoomLevelSet = new BingMapsZoomLevelSet();

            // Here the constructor of PointShape is PointShape(longitude, latitude)
            // So we change the order of the parameter to match our parameter order.
            // The default extent has the same parameter order issue.
            PointShape thinkGeoCoordinate = new PointShape(153.5084, -28.5537);
            RectangleShape defaultExtent = new RectangleShape(151.209389, -33.430000, 151.219389, -34.29000);

            // The extent and marker location are both in decimal degree,
            // We should use project to convert the default WGS84 coordinate to Spherical Mercator.
            ManagedProj4Projection wgs84ToBingProjection = new ManagedProj4Projection();
            wgs84ToBingProjection.InternalProjectionParametersString = ManagedProj4Projection.GetWgs84ParametersString();
            wgs84ToBingProjection.ExternalProjectionParametersString = ManagedProj4Projection.GetBingMapParametersString();
            wgs84ToBingProjection.Open();
            thinkGeoCoordinate = (PointShape)wgs84ToBingProjection.ConvertToExternalProjection(thinkGeoCoordinate);
            defaultExtent = wgs84ToBingProjection.ConvertToExternalProjection(defaultExtent);
            wgs84ToBingProjection.Close();

            Marker thinkGeoLocation = new Marker();
            thinkGeoLocation.Position = thinkGeoCoordinate;
            thinkGeoLocation.OffsetX = -22;
            // Here we put an image as boundle resource in the resource folder. 
            // So that the iOS image can find an image source for it.
            thinkGeoLocation.SetImage(UIImage.FromBundle("Pin"), UIControlState.Normal);
            MarkerOverlay markerOverlay = new MarkerOverlay();
            markerOverlay.Markers.Add(thinkGeoLocation);
            iOSMap.Overlays.Add("markerOverlay", markerOverlay);

            BingMapsTileOverlay bingMapOverlay = new BingMapsTileOverlay("Your application id");
            iOSMap.Overlays.Add(bingMapOverlay);

            string rootPath = Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments);
            bingMapOverlay.MapType = BingMapsMapType.AerialWithLabels;
            bingMapOverlay.TileCache = new FileBitmapTileCache(rootPath + "/CacheImages", "Road");
            bingMapOverlay.TileCache.TileMatrix.BoundingBoxUnit = GeographyUnit.Meter;
            bingMapOverlay.TileCache.TileMatrix.BoundingBox = bingMapOverlay.GetBoundingBox();
            bingMapOverlay.TileCache.ImageFormat = TileImageFormat.Png;
            iOSMap.CurrentExtent = defaultExtent;

            UIButton button = UIButton.FromType(UIButtonType.System);
            button.Frame = new RectangleF(0, 18, 120, 44);
            button.SetTitle("ZoomToMarker", UIControlState.Normal);
            button.BackgroundColor = UIColor.LightGray;
            button.TouchUpInside += (sender, e) =>
                {
                    iOSMap.ZoomTo(thinkGeoCoordinate.Buffer(500, GeographyUnit.Meter, DistanceUnit.Meter).GetBoundingBox());
                };

            View.AddSubview(iOSMap);
            View.AddSubview(button);

            iOSMap.Refresh();
        }
    }
}